@@ -24,6 +24,7 @@
 #include <linux/timer.h>
 #include <linux/kernel_stat.h>
 #include <linux/sched/signal.h>
+#include <linux/dma-mapping.h>
 
 #include <asm/ccwdev.h>
 #include <asm/cio.h>
@@ -148,19 +149,6 @@ static struct css_device_id io_subchannel_ids[] = {
 	{ /* end of list */ },
 };
 
-static int io_subchannel_prepare(struct subchannel *sch)
-{
-	struct ccw_device *cdev;
-	/*
-	 * Don't allow suspend while a ccw device registration
-	 * is still outstanding.
-	 */
-	cdev = sch_get_cdev(sch);
-	if (cdev && !device_is_registered(&cdev->dev))
-		return -EAGAIN;
-	return 0;
-}
-
 static int io_subchannel_settle(void)
 {
 	int ret;
@@ -185,7 +173,6 @@ static struct css_driver io_subchannel_driver = {
 	.probe = io_subchannel_probe,
 	.remove = io_subchannel_remove,
 	.shutdown = io_subchannel_shutdown,
-	.prepare = io_subchannel_prepare,
 	.settle = io_subchannel_settle,
 };
 
@@ -634,18 +621,10 @@ static const struct attribute_group *ccwdev_attr_groups[] = {
 	NULL,
 };
 
-static int ccw_device_add(struct ccw_device *cdev)
-{
-	struct device *dev = &cdev->dev;
-
-	dev->bus = &ccw_bus_type;
-	return device_add(dev);
-}
-
-static int match_dev_id(struct device *dev, void *data)
+static int match_dev_id(struct device *dev, const void *data)
 {
 	struct ccw_device *cdev = to_ccwdev(dev);
-	struct ccw_dev_id *dev_id = data;
+	struct ccw_dev_id *dev_id = (void *)data;
 
 	return ccw_dev_id_is_equal(&cdev->private->dev_id, dev_id);
 }
@@ -687,6 +666,9 @@ ccw_device_release(struct device *dev)
 	struct ccw_device *cdev;
 
 	cdev = to_ccwdev(dev);
+	cio_gp_dma_free(cdev->private->dma_pool, cdev->private->dma_area,
+			sizeof(*cdev->private->dma_area));
+	cio_gp_dma_destroy(cdev->private->dma_pool, &cdev->dev);
 	/* Release reference of parent subchannel. */
 	put_device(cdev->dev.parent);
 	kfree(cdev->private);
@@ -696,16 +678,48 @@ ccw_device_release(struct device *dev)
 static struct ccw_device * io_subchannel_allocate_dev(struct subchannel *sch)
 {
 	struct ccw_device *cdev;
+	struct gen_pool *dma_pool;
+	int ret;
 
 	cdev  = kzalloc(sizeof(*cdev), GFP_KERNEL);
-	if (cdev) {
-		cdev->private = kzalloc(sizeof(struct ccw_device_private),
-					GFP_KERNEL | GFP_DMA);
-		if (cdev->private)
-			return cdev;
+	if (!cdev) {
+		ret = -ENOMEM;
+		goto err_cdev;
 	}
+	cdev->private = kzalloc(sizeof(struct ccw_device_private),
+				GFP_KERNEL | GFP_DMA);
+	if (!cdev->private) {
+		ret = -ENOMEM;
+		goto err_priv;
+	}
+
+	cdev->dev.dma_mask = sch->dev.dma_mask;
+	ret = dma_set_coherent_mask(&cdev->dev, sch->dev.coherent_dma_mask);
+	if (ret)
+		goto err_coherent_mask;
+
+	dma_pool = cio_gp_dma_create(&cdev->dev, 1);
+	if (!dma_pool) {
+		ret = -ENOMEM;
+		goto err_dma_pool;
+	}
+	cdev->private->dma_pool = dma_pool;
+	cdev->private->dma_area = cio_gp_dma_zalloc(dma_pool, &cdev->dev,
+					sizeof(*cdev->private->dma_area));
+	if (!cdev->private->dma_area) {
+		ret = -ENOMEM;
+		goto err_dma_area;
+	}
+	return cdev;
+err_dma_area:
+	cio_gp_dma_destroy(dma_pool, &cdev->dev);
+err_dma_pool:
+err_coherent_mask:
+	kfree(cdev->private);
+err_priv:
 	kfree(cdev);
-	return ERR_PTR(-ENOMEM);
+err_cdev:
+	return ERR_PTR(ret);
 }
 
 static void ccw_device_todo(struct work_struct *work);
@@ -731,6 +745,7 @@ static int io_subchannel_initialize_dev(struct subchannel *sch,
 	cdev->ccwlock = sch->lock;
 	cdev->dev.parent = &sch->dev;
 	cdev->dev.release = ccw_device_release;
+	cdev->dev.bus = &ccw_bus_type;
 	cdev->dev.groups = ccwdev_attr_groups;
 	/* Do first half of device_register. */
 	device_initialize(&cdev->dev);
@@ -832,7 +847,7 @@ static void io_subchannel_register(struct ccw_device *cdev)
 		kobject_uevent(&sch->dev.kobj, KOBJ_ADD);
 	}
 	/* make it known to the system */
-	ret = ccw_device_add(cdev);
+	ret = device_add(&cdev->dev);
 	if (ret) {
 		CIO_MSG_EVENT(0, "Could not register ccw dev 0.%x.%04x: %d\n",
 			      cdev->private->dev_id.ssid,
@@ -886,7 +901,7 @@ io_subchannel_recog_done(struct ccw_device *cdev)
 			wake_up(&ccw_device_init_wq);
 		break;
 	case DEV_STATE_OFFLINE:
-		/* 
+		/*
 		 * We can't register the device in interrupt context so
 		 * we schedule a work item.
 		 */
@@ -1044,7 +1059,7 @@ static int io_subchannel_probe(struct subchannel *sch)
 			kobject_uevent(&sch->dev.kobj, KOBJ_ADD);
 		}
 		cdev = sch_get_cdev(sch);
-		rc = ccw_device_add(cdev);
+		rc = device_add(&cdev->dev);
 		if (rc) {
 			/* Release online reference. */
 			put_device(&cdev->dev);
@@ -1067,6 +1082,14 @@ static int io_subchannel_probe(struct subchannel *sch)
 	if (!io_priv)
 		goto out_schedule;
 
+	io_priv->dma_area = dma_alloc_coherent(&sch->dev,
+				sizeof(*io_priv->dma_area),
+				&io_priv->dma_area_dma, GFP_KERNEL);
+	if (!io_priv->dma_area) {
+		kfree(io_priv);
+		goto out_schedule;
+	}
+
 	set_io_private(sch, io_priv);
 	css_schedule_eval(sch->schid);
 	return 0;
@@ -1078,8 +1101,7 @@ out_schedule:
 	return 0;
 }
 
-static int
-io_subchannel_remove (struct subchannel *sch)
+static int io_subchannel_remove(struct subchannel *sch)
 {
 	struct io_subchannel_private *io_priv = to_io_private(sch);
 	struct ccw_device *cdev;
@@ -1087,15 +1109,15 @@ io_subchannel_remove (struct subchannel *sch)
 	cdev = sch_get_cdev(sch);
 	if (!cdev)
 		goto out_free;
-	io_subchannel_quiesce(sch);
-	/* Set ccw device to not operational and drop reference. */
-	spin_lock_irq(cdev->ccwlock);
+
+	ccw_device_unregister(cdev);
+	spin_lock_irq(sch->lock);
 	sch_set_cdev(sch, NULL);
 	set_io_private(sch, NULL);
-	cdev->private->state = DEV_STATE_NOT_OPER;
-	spin_unlock_irq(cdev->ccwlock);
-	ccw_device_unregister(cdev);
+	spin_unlock_irq(sch->lock);
 out_free:
+	dma_free_coherent(&sch->dev, sizeof(*io_priv->dma_area),
+			  io_priv->dma_area, io_priv->dma_area_dma);
 	kfree(io_priv);
 	sysfs_remove_group(&sch->dev.kobj, &io_subchannel_attr_group);
 	return 0;
@@ -1141,7 +1163,8 @@ static int io_subchannel_chp_event(struct subchannel *sch,
 				   struct chp_link *link, int event)
 {
 	struct ccw_device *cdev = sch_get_cdev(sch);
-	int mask;
+	int mask, chpid, valid_bit;
+	int path_event[8];
 
 	mask = chp_ssd_get_mask(&sch->ssd_info, link);
 	if (!mask)
@@ -1176,6 +1199,18 @@ static int io_subchannel_chp_event(struct subchannel *sch,
 			cdev->private->path_new_mask |= mask;
 		io_subchannel_verify(sch);
 		break;
+	case CHP_FCES_EVENT:
+		/* Forward Endpoint Security event */
+		for (chpid = 0, valid_bit = 0x80; chpid < 8; chpid++,
+				valid_bit >>= 1) {
+			if (mask & valid_bit)
+				path_event[chpid] = PE_PATH_FCES_EVENT;
+			else
+				path_event[chpid] = PE_NONE;
+		}
+		if (cdev && cdev->drv && cdev->drv->path_event)
+			cdev->drv->path_event(cdev, path_event);
+		break;
 	}
 	return 0;
 }
@@ -1238,7 +1273,7 @@ static int recovery_check(struct device *dev, void *data)
 		sch = to_subchannel(cdev->dev.parent);
 		if ((sch->schib.pmcw.pam & sch->opm) == sch->vpm)
 			break;
-		/* fall through */
+		fallthrough;
 	case DEV_STATE_DISCONNECTED:
 		CIO_MSG_EVENT(3, "recovery: trigger 0.%x.%04x\n",
 			      cdev->private->dev_id.ssid,
@@ -1393,7 +1428,7 @@ static enum io_sch_action sch_get_action(struct subchannel *sch)
 	}
 	if (device_is_disconnected(cdev))
 		return IO_SCH_REPROBE;
-	if (cdev->online && !cdev->private->flags.resuming)
+	if (cdev->online)
 		return IO_SCH_VERIFY;
 	if (cdev->private->state == DEV_STATE_NOT_OPER)
 		return IO_SCH_UNREG_ATTACH;
@@ -1485,11 +1520,6 @@ static int io_subchannel_sch_event(struct subchannel *sch, int process)
 		break;
 	case IO_SCH_UNREG_ATTACH:
 		spin_lock_irqsave(sch->lock, flags);
-		if (cdev->private->flags.resuming) {
-			/* Device will be handled later. */
-			rc = 0;
-			goto out_unlock;
-		}
 		sch_set_cdev(sch, NULL);
 		spin_unlock_irqrestore(sch->lock, flags);
 		/* Unregister ccw device. */
@@ -1502,8 +1532,7 @@ static int io_subchannel_sch_event(struct subchannel *sch, int process)
 	switch (action) {
 	case IO_SCH_ORPH_UNREG:
 	case IO_SCH_UNREG:
-		if (!cdev || !cdev->private->flags.resuming)
-			css_sch_device_unregister(sch);
+		css_sch_device_unregister(sch);
 		break;
 	case IO_SCH_ORPH_ATTACH:
 	case IO_SCH_UNREG_ATTACH:
@@ -1601,13 +1630,19 @@ struct ccw_device * __init ccw_device_create_console(struct ccw_driver *drv)
 		return ERR_CAST(sch);
 
 	io_priv = kzalloc(sizeof(*io_priv), GFP_KERNEL | GFP_DMA);
-	if (!io_priv) {
-		put_device(&sch->dev);
-		return ERR_PTR(-ENOMEM);
-	}
+	if (!io_priv)
+		goto err_priv;
+	io_priv->dma_area = dma_alloc_coherent(&sch->dev,
+				sizeof(*io_priv->dma_area),
+				&io_priv->dma_area_dma, GFP_KERNEL);
+	if (!io_priv->dma_area)
+		goto err_dma_area;
 	set_io_private(sch, io_priv);
 	cdev = io_subchannel_create_ccwdev(sch);
 	if (IS_ERR(cdev)) {
+		dma_free_coherent(&sch->dev, sizeof(*io_priv->dma_area),
+				  io_priv->dma_area, io_priv->dma_area_dma);
+		set_io_private(sch, NULL);
 		put_device(&sch->dev);
 		kfree(io_priv);
 		return cdev;
@@ -1615,6 +1650,12 @@ struct ccw_device * __init ccw_device_create_console(struct ccw_driver *drv)
 	cdev->drv = drv;
 	ccw_device_set_int_class(cdev);
 	return cdev;
+
+err_dma_area:
+	kfree(io_priv);
+err_priv:
+	put_device(&sch->dev);
+	return ERR_PTR(-ENOMEM);
 }
 
 void __init ccw_device_destroy_console(struct ccw_device *cdev)
@@ -1623,6 +1664,8 @@ void __init ccw_device_destroy_console(struct ccw_device *cdev)
 	struct io_subchannel_private *io_priv = to_io_private(sch);
 
 	set_io_private(sch, NULL);
+	dma_free_coherent(&sch->dev, sizeof(*io_priv->dma_area),
+			  io_priv->dma_area, io_priv->dma_area_dma);
 	put_device(&sch->dev);
 	put_device(&cdev->dev);
 	kfree(io_priv);
@@ -1644,33 +1687,11 @@ void ccw_device_wait_idle(struct ccw_device *cdev)
 		cio_tsch(sch);
 		if (sch->schib.scsw.cmd.actl == 0)
 			break;
-		udelay_simple(100);
+		udelay(100);
 	}
 }
-
-static int ccw_device_pm_restore(struct device *dev);
-
-int ccw_device_force_console(struct ccw_device *cdev)
-{
-	return ccw_device_pm_restore(&cdev->dev);
-}
-EXPORT_SYMBOL_GPL(ccw_device_force_console);
 #endif
 
-/*
- * get ccw_device matching the busid, but only if owned by cdrv
- */
-static int
-__ccwdev_check_busid(struct device *dev, void *id)
-{
-	char *bus_id;
-
-	bus_id = id;
-
-	return (strcmp(bus_id, dev_name(dev)) == 0);
-}
-
-
 /**
  * get_ccwdev_by_busid() - obtain device from a bus id
  * @cdrv: driver the device is owned by
@@ -1687,8 +1708,7 @@ struct ccw_device *get_ccwdev_by_busid(struct ccw_driver *cdrv,
 {
 	struct device *dev;
 
-	dev = driver_find_device(&cdrv->driver, NULL, (void *)bus_id,
-				 __ccwdev_check_busid);
+	dev = driver_find_device_by_name(&cdrv->driver, bus_id);
 
 	return dev ? to_ccwdev(dev) : NULL;
 }
@@ -1726,6 +1746,7 @@ static int ccw_device_remove(struct device *dev)
 {
 	struct ccw_device *cdev = to_ccwdev(dev);
 	struct ccw_driver *cdrv = cdev->drv;
+	struct subchannel *sch;
 	int ret;
 
 	if (cdrv->remove)
@@ -1751,7 +1772,9 @@ static int ccw_device_remove(struct device *dev)
 	ccw_device_set_timeout(cdev, 0);
 	cdev->drv = NULL;
 	cdev->private->int_class = IRQIO_CIO;
+	sch = to_subchannel(cdev->dev.parent);
 	spin_unlock_irq(cdev->ccwlock);
+	io_subchannel_quiesce(sch);
 	__disable_cmf(cdev);
 
 	return 0;
@@ -1767,235 +1790,6 @@ static void ccw_device_shutdown(struct device *dev)
 	__disable_cmf(cdev);
 }
 
-static int ccw_device_pm_prepare(struct device *dev)
-{
-	struct ccw_device *cdev = to_ccwdev(dev);
-
-	if (work_pending(&cdev->private->todo_work))
-		return -EAGAIN;
-	/* Fail while device is being set online/offline. */
-	if (atomic_read(&cdev->private->onoff))
-		return -EAGAIN;
-
-	if (cdev->online && cdev->drv && cdev->drv->prepare)
-		return cdev->drv->prepare(cdev);
-
-	return 0;
-}
-
-static void ccw_device_pm_complete(struct device *dev)
-{
-	struct ccw_device *cdev = to_ccwdev(dev);
-
-	if (cdev->online && cdev->drv && cdev->drv->complete)
-		cdev->drv->complete(cdev);
-}
-
-static int ccw_device_pm_freeze(struct device *dev)
-{
-	struct ccw_device *cdev = to_ccwdev(dev);
-	struct subchannel *sch = to_subchannel(cdev->dev.parent);
-	int ret, cm_enabled;
-
-	/* Fail suspend while device is in transistional state. */
-	if (!dev_fsm_final_state(cdev))
-		return -EAGAIN;
-	if (!cdev->online)
-		return 0;
-	if (cdev->drv && cdev->drv->freeze) {
-		ret = cdev->drv->freeze(cdev);
-		if (ret)
-			return ret;
-	}
-
-	spin_lock_irq(sch->lock);
-	cm_enabled = cdev->private->cmb != NULL;
-	spin_unlock_irq(sch->lock);
-	if (cm_enabled) {
-		/* Don't have the css write on memory. */
-		ret = ccw_set_cmf(cdev, 0);
-		if (ret)
-			return ret;
-	}
-	/* From here on, disallow device driver I/O. */
-	spin_lock_irq(sch->lock);
-	ret = cio_disable_subchannel(sch);
-	spin_unlock_irq(sch->lock);
-
-	return ret;
-}
-
-static int ccw_device_pm_thaw(struct device *dev)
-{
-	struct ccw_device *cdev = to_ccwdev(dev);
-	struct subchannel *sch = to_subchannel(cdev->dev.parent);
-	int ret, cm_enabled;
-
-	if (!cdev->online)
-		return 0;
-
-	spin_lock_irq(sch->lock);
-	/* Allow device driver I/O again. */
-	ret = cio_enable_subchannel(sch, (u32)(addr_t)sch);
-	cm_enabled = cdev->private->cmb != NULL;
-	spin_unlock_irq(sch->lock);
-	if (ret)
-		return ret;
-
-	if (cm_enabled) {
-		ret = ccw_set_cmf(cdev, 1);
-		if (ret)
-			return ret;
-	}
-
-	if (cdev->drv && cdev->drv->thaw)
-		ret = cdev->drv->thaw(cdev);
-
-	return ret;
-}
-
-static void __ccw_device_pm_restore(struct ccw_device *cdev)
-{
-	struct subchannel *sch = to_subchannel(cdev->dev.parent);
-
-	spin_lock_irq(sch->lock);
-	if (cio_is_console(sch->schid)) {
-		cio_enable_subchannel(sch, (u32)(addr_t)sch);
-		goto out_unlock;
-	}
-	/*
-	 * While we were sleeping, devices may have gone or become
-	 * available again. Kick re-detection.
-	 */
-	cdev->private->flags.resuming = 1;
-	cdev->private->path_new_mask = LPM_ANYPATH;
-	css_sched_sch_todo(sch, SCH_TODO_EVAL);
-	spin_unlock_irq(sch->lock);
-	css_wait_for_slow_path();
-
-	/* cdev may have been moved to a different subchannel. */
-	sch = to_subchannel(cdev->dev.parent);
-	spin_lock_irq(sch->lock);
-	if (cdev->private->state != DEV_STATE_ONLINE &&
-	    cdev->private->state != DEV_STATE_OFFLINE)
-		goto out_unlock;
-
-	ccw_device_recognition(cdev);
-	spin_unlock_irq(sch->lock);
-	wait_event(cdev->private->wait_q, dev_fsm_final_state(cdev) ||
-		   cdev->private->state == DEV_STATE_DISCONNECTED);
-	spin_lock_irq(sch->lock);
-
-out_unlock:
-	cdev->private->flags.resuming = 0;
-	spin_unlock_irq(sch->lock);
-}
-
-static int resume_handle_boxed(struct ccw_device *cdev)
-{
-	cdev->private->state = DEV_STATE_BOXED;
-	if (ccw_device_notify(cdev, CIO_BOXED) == NOTIFY_OK)
-		return 0;
-	ccw_device_sched_todo(cdev, CDEV_TODO_UNREG);
-	return -ENODEV;
-}
-
-static int resume_handle_disc(struct ccw_device *cdev)
-{
-	cdev->private->state = DEV_STATE_DISCONNECTED;
-	if (ccw_device_notify(cdev, CIO_GONE) == NOTIFY_OK)
-		return 0;
-	ccw_device_sched_todo(cdev, CDEV_TODO_UNREG);
-	return -ENODEV;
-}
-
-static int ccw_device_pm_restore(struct device *dev)
-{
-	struct ccw_device *cdev = to_ccwdev(dev);
-	struct subchannel *sch;
-	int ret = 0;
-
-	__ccw_device_pm_restore(cdev);
-	sch = to_subchannel(cdev->dev.parent);
-	spin_lock_irq(sch->lock);
-	if (cio_is_console(sch->schid))
-		goto out_restore;
-
-	/* check recognition results */
-	switch (cdev->private->state) {
-	case DEV_STATE_OFFLINE:
-	case DEV_STATE_ONLINE:
-		cdev->private->flags.donotify = 0;
-		break;
-	case DEV_STATE_BOXED:
-		ret = resume_handle_boxed(cdev);
-		if (ret)
-			goto out_unlock;
-		goto out_restore;
-	default:
-		ret = resume_handle_disc(cdev);
-		if (ret)
-			goto out_unlock;
-		goto out_restore;
-	}
-	/* check if the device type has changed */
-	if (!ccw_device_test_sense_data(cdev)) {
-		ccw_device_update_sense_data(cdev);
-		ccw_device_sched_todo(cdev, CDEV_TODO_REBIND);
-		ret = -ENODEV;
-		goto out_unlock;
-	}
-	if (!cdev->online)
-		goto out_unlock;
-
-	if (ccw_device_online(cdev)) {
-		ret = resume_handle_disc(cdev);
-		if (ret)
-			goto out_unlock;
-		goto out_restore;
-	}
-	spin_unlock_irq(sch->lock);
-	wait_event(cdev->private->wait_q, dev_fsm_final_state(cdev));
-	spin_lock_irq(sch->lock);
-
-	if (ccw_device_notify(cdev, CIO_OPER) == NOTIFY_BAD) {
-		ccw_device_sched_todo(cdev, CDEV_TODO_UNREG);
-		ret = -ENODEV;
-		goto out_unlock;
-	}
-
-	/* reenable cmf, if needed */
-	if (cdev->private->cmb) {
-		spin_unlock_irq(sch->lock);
-		ret = ccw_set_cmf(cdev, 1);
-		spin_lock_irq(sch->lock);
-		if (ret) {
-			CIO_MSG_EVENT(2, "resume: cdev 0.%x.%04x: cmf failed "
-				      "(rc=%d)\n", cdev->private->dev_id.ssid,
-				      cdev->private->dev_id.devno, ret);
-			ret = 0;
-		}
-	}
-
-out_restore:
-	spin_unlock_irq(sch->lock);
-	if (cdev->online && cdev->drv && cdev->drv->restore)
-		ret = cdev->drv->restore(cdev);
-	return ret;
-
-out_unlock:
-	spin_unlock_irq(sch->lock);
-	return ret;
-}
-
-static const struct dev_pm_ops ccw_pm_ops = {
-	.prepare = ccw_device_pm_prepare,
-	.complete = ccw_device_pm_complete,
-	.freeze = ccw_device_pm_freeze,
-	.thaw = ccw_device_pm_thaw,
-	.restore = ccw_device_pm_restore,
-};
-
 static struct bus_type ccw_bus_type = {
 	.name   = "ccw",
 	.match  = ccw_bus_match,
@@ -2003,7 +1797,6 @@ static struct bus_type ccw_bus_type = {
 	.probe  = ccw_device_probe,
 	.remove = ccw_device_remove,
 	.shutdown = ccw_device_shutdown,
-	.pm = &ccw_pm_ops,
 };
 
 /**
@@ -2065,7 +1858,7 @@ static void ccw_device_todo(struct work_struct *work)
 	case CDEV_TODO_UNREG_EVAL:
 		if (!sch_is_pseudo_sch(sch))
 			css_schedule_eval(sch->schid);
-		/* fall-through */
+		fallthrough;
 	case CDEV_TODO_UNREG:
 		if (sch_is_pseudo_sch(sch))
 			ccw_device_unregister(cdev);
