@@ -1,5 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0
 /*
- * Copyright (C) 2005-2017 Junjiro R. Okajima
+ * Copyright (C) 2005-2020 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +20,7 @@
  * lookup and dentry operations
  */
 
+#include <linux/iversion.h>
 #include <linux/namei.h>
 #include "aufs.h"
 
@@ -33,6 +35,7 @@ au_do_lookup(struct dentry *h_parent, struct dentry *dentry,
 	struct dentry *h_dentry;
 	struct inode *h_inode;
 	struct au_branch *br;
+	struct user_namespace *h_userns;
 	int wh_found, opq;
 	unsigned char wh_able;
 	const unsigned char allow_neg = !!au_ftest_lkup(args->flags, ALLOW_NEG);
@@ -41,9 +44,11 @@ au_do_lookup(struct dentry *h_parent, struct dentry *dentry,
 
 	wh_found = 0;
 	br = au_sbr(dentry->d_sb, bindex);
+	h_userns = au_br_userns(br);
 	wh_able = !!au_br_whable(br->br_perm);
 	if (wh_able)
-		wh_found = au_wh_test(h_parent, &args->whname, ignore_perm);
+		wh_found = au_wh_test(h_userns, h_parent, &args->whname,
+				      ignore_perm);
 	h_dentry = ERR_PTR(wh_found);
 	if (!wh_found)
 		goto real_lookup;
@@ -60,7 +65,7 @@ real_lookup:
 	if (!ignore_perm)
 		h_dentry = vfsub_lkup_one(args->name, h_parent);
 	else
-		h_dentry = au_sio_lkup_one(args->name, h_parent);
+		h_dentry = au_sio_lkup_one(h_userns, args->name, h_parent);
 	if (IS_ERR(h_dentry)) {
 		if (PTR_ERR(h_dentry) == -ENAMETOOLONG
 		    && !allow_neg)
@@ -94,8 +99,8 @@ real_lookup:
 	    || (d_really_is_positive(dentry) && !d_is_dir(dentry)))
 		goto out; /* success */
 
-	vfsub_inode_lock_shared_nested(h_inode, AuLsc_I_CHILD);
-	opq = au_diropq_test(h_dentry);
+	inode_lock_shared_nested(h_inode, AuLsc_I_CHILD);
+	opq = au_diropq_test(h_userns, h_dentry);
 	inode_unlock_shared(h_inode);
 	if (opq > 0)
 		au_set_dbdiropq(dentry, bindex);
@@ -179,7 +184,7 @@ int au_lkup_dentry(struct dentry *dentry, aufs_bindex_t btop,
 		}
 
 		h_dir = d_inode(h_parent);
-		vfsub_inode_lock_shared_nested(h_dir, AuLsc_I_PARENT);
+		inode_lock_shared_nested(h_dir, AuLsc_I_PARENT);
 		h_dentry = au_do_lookup(h_parent, dentry, bindex, &args);
 		inode_unlock_shared(h_dir);
 		err = PTR_ERR(h_dentry);
@@ -233,19 +238,20 @@ int au_lkup_dentry(struct dentry *dentry, aufs_bindex_t btop,
 
 out_parent:
 	dput(parent);
-	kfree(args.whname.name);
+	au_kfree_try_rcu(args.whname.name);
 	if (dirren)
 		au_dr_lkup_fin(&args);
 out:
 	return err;
 }
 
-struct dentry *au_sio_lkup_one(struct qstr *name, struct dentry *parent)
+struct dentry *au_sio_lkup_one(struct user_namespace *userns, struct qstr *name,
+			       struct dentry *parent)
 {
 	struct dentry *dentry;
 	int wkq_err;
 
-	if (!au_test_h_perm_sio(d_inode(parent), MAY_EXEC))
+	if (!au_test_h_perm_sio(userns, d_inode(parent), MAY_EXEC))
 		dentry = vfsub_lkup_one(name, parent);
 	else {
 		struct vfsub_lkup_one_args args = {
@@ -270,14 +276,16 @@ int au_lkup_neg(struct dentry *dentry, aufs_bindex_t bindex, int wh)
 	int err;
 	struct dentry *parent, *h_parent, *h_dentry;
 	struct au_branch *br;
+	struct user_namespace *h_userns;
 
 	parent = dget_parent(dentry);
 	h_parent = au_h_dptr(parent, bindex);
 	br = au_sbr(dentry->d_sb, bindex);
+	h_userns = au_br_userns(br);
 	if (wh)
 		h_dentry = au_whtmp_lkup(h_parent, br, &dentry->d_name);
 	else
-		h_dentry = au_sio_lkup_one(&dentry->d_name, h_parent);
+		h_dentry = au_sio_lkup_one(h_userns, &dentry->d_name, h_parent);
 	err = PTR_ERR(h_dentry);
 	if (IS_ERR(h_dentry))
 		goto out;
@@ -322,7 +330,7 @@ static void au_iattr_save(struct au_iattr *ia, struct inode *h_inode)
 	/* ia->i_nlink = h_inode->i_nlink; */
 	ia->i_uid = h_inode->i_uid;
 	ia->i_gid = h_inode->i_gid;
-	ia->i_version = h_inode->i_version;
+	ia->i_version = inode_query_iversion(h_inode);
 /*
 	ia->i_size = h_inode->i_size;
 	ia->i_blocks = h_inode->i_blocks;
@@ -336,7 +344,7 @@ static int au_iattr_test(struct au_iattr *ia, struct inode *h_inode)
 		/* || ia->i_nlink != h_inode->i_nlink */
 		|| !uid_eq(ia->i_uid, h_inode->i_uid)
 		|| !gid_eq(ia->i_gid, h_inode->i_gid)
-		|| ia->i_version != h_inode->i_version
+		|| !inode_eq_iversion(h_inode, ia->i_version)
 /*
 		|| ia->i_size != h_inode->i_size
 		|| ia->i_blocks != h_inode->i_blocks
@@ -607,7 +615,7 @@ static int au_refresh_by_dinfo(struct dentry *dentry, struct au_dinfo *dinfo,
 	if (d_really_is_positive(dentry))
 		inode = d_inode(dentry);
 	if (!orig_h.inode) {
-		AuDbg("nagative originally\n");
+		AuDbg("negative originally\n");
 		if (inode) {
 			au_hide(dentry);
 			goto out;
